import { Reader } from '../../util/buffers/Reader';
import { RespAttributes, RespPush } from './extensions';
export class RespDecoder {
    reader;
    constructor(reader = new Reader()) {
        this.reader = reader;
    }
    read(uint8) {
        this.reader.reset(uint8);
        return this.val();
    }
    decode(uint8) {
        this.reader.reset(uint8);
        return this.val();
    }
    val() {
        const reader = this.reader;
        const type = reader.u8();
        switch (type) {
            case 58:
                return this.readInt();
            case 44:
                return this.readFloat();
            case 43:
                return this.readStrSimple();
            case 61:
                return this.readStrVerbatim();
            case 35:
                return this.readBool();
            case 95:
                return (reader.x += 2), null;
            case 36:
                return this.readStrBulk();
            case 37:
                return this.readObj();
            case 42:
                return this.readArr();
            case 62:
                return new RespPush(this.readArr());
            case 40:
                return this.readBigint();
            case 126:
                return this.readSet();
            case 45:
                return this.readErrSimple();
            case 33:
                return this.readErrBulk();
            case 124:
                return new RespAttributes(this.readObj());
        }
        throw new Error('UNKNOWN_TYPE');
    }
    readMinorLen(minor) {
        throw new Error('Not implemented');
    }
    readLength() {
        const reader = this.reader;
        const uint8 = reader.uint8;
        const x = reader.x;
        let number = 0;
        for (let i = x;; i++) {
            const c = uint8[i];
            if (c === 13) {
                reader.x = i + 2;
                return number;
            }
            number = number * 10 + (c - 48);
        }
    }
    readBool() {
        const reader = this.reader;
        const x = reader.x;
        const c = reader.uint8[x];
        reader.x = x + 3;
        return c === 116;
    }
    readInt() {
        const reader = this.reader;
        const uint8 = reader.uint8;
        let x = reader.x;
        let negative = false;
        let c = uint8[x];
        let number = 0;
        if (c === 45) {
            negative = true;
            x++;
        }
        else if (c === 43)
            x++;
        for (let i = x;; i++) {
            c = uint8[i];
            if (c === 13) {
                reader.x = i + 2;
                return negative ? -number : number;
            }
            number = number * 10 + (c - 48);
        }
    }
    readFloat() {
        const reader = this.reader;
        const uint8 = reader.uint8;
        const x = reader.x;
        let c = uint8[x];
        for (let i = x;; i++) {
            c = uint8[i];
            if (c === 13) {
                const length = i - x;
                const str = reader.ascii(length);
                switch (length) {
                    case 3:
                        switch (str) {
                            case 'inf':
                                reader.x = i + 2;
                                return Infinity;
                            case 'nan':
                                reader.x = i + 2;
                                return NaN;
                        }
                        break;
                    case 4:
                        if (str === '-inf') {
                            reader.x = i + 2;
                            return -Infinity;
                        }
                        break;
                }
                reader.x = i + 2;
                return Number(str);
            }
        }
    }
    readBigint() {
        const reader = this.reader;
        const uint8 = reader.uint8;
        const x = reader.x;
        let c = uint8[x];
        for (let i = x;; i++) {
            c = uint8[i];
            if (c === 13) {
                const str = reader.ascii(i - x);
                reader.x = i + 2;
                return BigInt(str);
            }
        }
    }
    readStrSimple() {
        const reader = this.reader;
        const uint8 = reader.uint8;
        const x = reader.x;
        for (let i = x; i < uint8.length; i++) {
            if (uint8[i] !== 13)
                continue;
            const str = reader.utf8(i - reader.x);
            reader.x = i + 2;
            return str;
        }
        throw new Error('INVALID_STR');
    }
    readStrVerbatim() {
        const reader = this.reader;
        const length = this.readLength();
        const encoding = reader.utf8(3);
        reader.x++;
        if (encoding === 'txt') {
            const str = reader.utf8(length);
            reader.x += 2;
            return str;
        }
        const buf = reader.buf(length);
        reader.x += 2;
        return buf;
    }
    readStrBulk() {
        const reader = this.reader;
        const length = this.readLength();
        const buf = reader.buf(length);
        reader.x += 2;
        return buf;
    }
    readErrSimple() {
        const reader = this.reader;
        const uint8 = reader.uint8;
        const x = reader.x;
        for (let i = x; i < uint8.length; i++) {
            if (uint8[i] !== 13)
                continue;
            const message = reader.utf8(i - reader.x);
            reader.x = i + 2;
            return new Error(message);
        }
        throw new Error('INVALID_ERR');
    }
    readErrBulk() {
        const reader = this.reader;
        const length = this.readLength();
        const message = reader.utf8(length);
        reader.x += 2;
        return new Error(message);
    }
    readArr() {
        const length = this.readLength();
        const arr = [];
        for (let i = 0; i < length; i++)
            arr.push(this.val());
        return arr;
    }
    readSet() {
        const length = this.readLength();
        const set = new Set();
        for (let i = 0; i < length; i++)
            set.add(this.val());
        return set;
    }
    readObj() {
        const length = this.readLength();
        const obj = {};
        for (let i = 0; i < length; i++) {
            const key = this.val() + '';
            obj[key] = this.val();
        }
        return obj;
    }
}
